@@ -38,7 +38,7 @@ public:
     KStandardItemListWidgetInformant();
     virtual ~KStandardItemListWidgetInformant();
 
-    virtual QSizeF itemSizeHint(int index, const KItemListView* view) const;
+    virtual void calculateItemSizeHints(QVector<QSizeF>& sizeHints, const KItemListView* view) const;
 
     virtual qreal preferredRoleColumnWidth(const QByteArray& role,
                                            int index,
@@ -53,6 +53,13 @@ protected:
      */
     virtual QString itemText(int index, const KItemListView* view) const;
 
+    /**
+     * @return The value of the "isLink" role. The default implementation returns false.
+     *         The derived class should reimplement this function, when information about
+     *         links is available and in usage.
+     */
+    virtual bool itemIsLink(int index, const KItemListView* view) const;
+
     /**
      * @return String representation of the role \a role. The representation of
      *         a role might depend on other roles, so the values of all roles
@@ -61,6 +68,15 @@ protected:
     virtual QString roleText(const QByteArray& role,
                              const QHash<QByteArray, QVariant>& values) const;
 
+    /**
+    * @return A font based on baseFont which is customized for symlinks.
+    */
+    virtual QFont customizedFontForLinks(const QFont& baseFont) const;
+
+    void calculateIconsLayoutItemSizeHints(QVector<QSizeF>& sizeHints, const KItemListView* view) const;
+    void calculateCompactLayoutItemSizeHints(QVector<QSizeF>& sizeHints, const KItemListView* view) const;
+    void calculateDetailsLayoutItemSizeHints(QVector<QSizeF>& sizeHints, const KItemListView* view) const;
+
     friend class KStandardItemListWidget; // Accesses roleText()
 };
 
