@@ -8,9 +8,9 @@
 #ifndef HTMLOPTS_H
 #define HTMLOPTS_H
 
-#include <QtGui/QCheckBox>
-#include <QtGui/QLineEdit>
-#include <QtGui/QComboBox>
+#include <QCheckBox>
+#include <QLineEdit>
+#include <QComboBox>
 
 //-----------------------------------------------------------------------------
 // The "Misc Options" Tab for the HTML view contains :
@@ -51,6 +51,7 @@ private:
     QCheckBox* m_pOnlyMarkedBookmarksCheckBox;
     QCheckBox* m_pAccessKeys;
     QCheckBox* m_pDoNotTrack;
+    QCheckBox* m_pOfferToSaveWebsitePassword;
     KIntNumInput* m_pMaxFormCompletionItems;
 };
 
