@@ -89,6 +89,9 @@ K_EXPORT_PLASMA_APPLET(folderview, FolderView)
 Q_DECLARE_METATYPE(Qt::SortOrder)
 Q_DECLARE_METATYPE(ProxyModel::FilterMode)
 Q_DECLARE_METATYPE(IconView::Flow)
+Q_DECLARE_METATYPE(IconView::Layout)
+Q_DECLARE_METATYPE(IconView::Alignment)
+Q_DECLARE_METATYPE(FolderView::LabelType)
 
 MimeModel::MimeModel(QObject *parent)
     : QStringListModel(parent)
@@ -372,8 +375,7 @@ void FolderView::init()
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), SLOT(plasmaThemeChanged()));
 
     // Find out about network availability changes
-    connect(Solid::Networking::notifier(), SIGNAL(statusChanged(Solid::Networking::Status)),
-            SLOT(networkStatusChanged(Solid::Networking::Status)));
+    connect(Solid::Networking::notifier(), SIGNAL(shouldConnect()), SLOT(networkAvailable()));
 
     KConfigGroup cg = config();
     m_customLabel         = cg.readEntry("customLabel", "");
@@ -390,24 +392,22 @@ void FolderView::init()
     m_sortColumn          = cg.readEntry("sortColumn", int(KDirModel::Name));
     m_sortOrder           = sortOrderStringToEnum(cg.readEntry("sortOrder", "ascending"));
     m_filterFiles         = cg.readEntry("filterFiles", "*");
-    m_filterType          = static_cast<ProxyModel::FilterMode>(cg.readEntry("filter", 0));
+    m_filterType          = static_cast<ProxyModel::FilterMode>(cg.readEntry("filter", static_cast<int>(ProxyModel::NoFilter)));
     m_filterFilesMimeList = cg.readEntry("mimeFilter", QStringList());
-    m_blankLabel          = cg.readEntry("blankLabel", false);
+    m_labelType           = static_cast<FolderView::LabelType>(cg.readEntry("labelType", static_cast<int>(FolderView::None)));
     m_showSelectionMarker = KGlobalSettings::singleClick();
 
-    if (isContainment()) {
-        m_flow = layoutDirection() == Qt::LeftToRight ? IconView::TopToBottom : IconView::TopToBottomRightToLeft;
-    } else {
-        m_flow = layoutDirection() == Qt::LeftToRight ? IconView::LeftToRight : IconView::RightToLeft;
-    }
-    m_flow = static_cast<IconView::Flow>(cg.readEntry("flow", static_cast<int>(m_flow)));
+    m_layout = isContainment() ? IconView::Columns : IconView::Rows;
+    m_alignment = layoutDirection() == Qt::LeftToRight ? IconView::Left : IconView::Right;
+    m_layout = static_cast<IconView::Layout>(cg.readEntry("layout", static_cast<int>(m_layout)));
+    m_alignment = static_cast<IconView::Alignment>(cg.readEntry("alignment", static_cast<int>(m_alignment)));
 
     m_model->setFilterMode(m_filterType);
     m_model->setMimeTypeFilterList(m_filterFilesMimeList);
     m_model->setFileNameFilter(m_filterFiles);
     m_model->setSortDirectoriesFirst(m_sortDirsFirst);
-    m_model->setDynamicSortFilter(m_sortColumn != -1);
-    m_model->sort(m_sortColumn != -1 ? m_sortColumn : KDirModel::Name, m_sortOrder);
+    m_model->setDynamicSortFilter(m_sortColumn != int(FolderView::Unsorted));
+    m_model->sort(m_sortColumn != int(FolderView::Unsorted) ? m_sortColumn : KDirModel::Name, m_sortOrder);
 
     m_dirLister = new DirLister(this);
     m_dirLister->setDelayedMimeTypes(true);
@@ -477,19 +477,19 @@ void FolderView::configChanged()
     bool needReload = false;
     bool preserveIconPositions = false;
 
+    const FolderView::LabelType labelType = static_cast<FolderView::LabelType>(cg.readEntry("labelType", static_cast<int>(m_labelType)));
+    if (labelType != m_labelType) {
+        m_labelType = labelType;
+        needReload = true;
+    }
+
     //Reload m_customLabel values
     const QString label = cg.readEntry("customLabel", m_customLabel);
     if (label != m_customLabel) {
         m_customLabel = label;
-        setAppletTitle();
         needReload = true;
     }
 
-    const bool blank = cg.readEntry("blankLabel", m_blankLabel);
-    if (blank != m_blankLabel) {
-        m_blankLabel = blank;
-        needReload = true;
-    }
     //Reload m_customIconSize values
     m_customIconSize = cg.readEntry("customIconSize", m_customIconSize);
     if (m_customIconSize != iconSize().width()) {
@@ -548,9 +548,13 @@ void FolderView::configChanged()
         m_sortDirsFirst = sortDirsFirst;
 
         m_model->setSortDirectoriesFirst(m_sortDirsFirst);
-        if (m_sortColumn != -1) {
+        if (m_sortColumn != int(FolderView::Unsorted)) {
             m_model->invalidate();
         }
+
+        if (QAction *action = m_actionCollection.action("dirs_first")) {
+            action->setChecked(m_sortDirsFirst);
+        }
     }
 
     const int sortColumn = cg.readEntry("sortColumn", m_sortColumn);
@@ -558,7 +562,7 @@ void FolderView::configChanged()
     if ((m_sortColumn != sortColumn) || (m_sortOrder != sortOrder)) {
         m_sortColumn = sortColumn;
         m_sortOrder = sortOrder;
-        if (m_sortColumn != -1) {
+        if (m_sortColumn != int(FolderView::Unsorted)) {
             m_model->invalidate();
             m_model->sort(m_sortColumn, m_sortOrder);
             m_model->setDynamicSortFilter(true);
@@ -569,9 +573,12 @@ void FolderView::configChanged()
         updateSortActionsState();
     }
 
-    const int flow = static_cast<IconView::Flow>(cg.readEntry("flow", static_cast<int>(m_flow)));
-    if (flow != m_flow) {
-        m_flow = static_cast<IconView::Flow>(flow);
+    const IconView::Layout layout = static_cast<IconView::Layout>(cg.readEntry("layout", static_cast<int>(m_layout)));
+    const IconView::Alignment alignment = static_cast<IconView::Alignment>(cg.readEntry("alignment", static_cast<int>(m_alignment)));
+    if ((m_layout != layout) || (m_alignment != alignment)) {
+        m_layout = layout;
+        m_alignment = alignment;
+        updateFlowActionsState();
     }
 
     const ProxyModel::FilterMode filterType = static_cast<ProxyModel::FilterMode>(cg.readEntry("filter", static_cast<int>(m_filterType)));
@@ -700,56 +707,42 @@ void FolderView::createConfigurationInterface(KConfigDialog *parent)
     pMimeModel->setSourceModel(mimeModel);
     uiFilter.filterFilesList->setModel(pMimeModel);
 
+    uiLocation.titleCombo->addItem(i18n("None"), QVariant::fromValue(FolderView::None));
+    uiLocation.titleCombo->addItem(i18n("Default"), QVariant::fromValue(FolderView::PlaceName));
+    uiLocation.titleCombo->addItem(i18n("Full Path"), QVariant::fromValue(FolderView::FullPath));
+    uiLocation.titleCombo->addItem(i18n("Custom title"), QVariant::fromValue(FolderView::Custom));
+
+    if (m_labelType == FolderView::Custom) {
+        uiLocation.titleEdit->setEnabled(true);
+        uiLocation.titleEdit->setText(m_customLabel);
+    } else {
+        uiLocation.titleEdit->setEnabled(false);
+    }
+
     // The label is not shown when the applet is acting as a containment,
     // so don't confuse the user by making it editable.
     if (isContainment()) {
-        uiDisplay.titleLabel->hide();
-        uiDisplay.titleEdit->hide();
-        uiDisplay.headerTitle->hide();
+        uiLocation.titleLabel->hide();
+        uiLocation.titleCombo->hide();
+        uiLocation.titleEdit->hide();
     }
 
-    KLineEdit *ledit = new KLineEdit(widgetDisplay);
-    ledit->setClearButtonShown(false);
-    ledit->setClickMessage(i18n("Title"));
-    uiDisplay.titleEdit->setLineEdit(ledit);
-
-    QString configTitleText = m_customLabel;
-    if (m_customLabel.isEmpty() || m_customLabel == "___EMPTY___") {
-        configTitleText = i18n("None");
-    } else if (m_customLabel == "___DEFAULT___") {
-        configTitleText = i18n("Default");
-    } else if (m_customLabel == "___FULL___") {
-        configTitleText = i18n("Full path");
-    } else {
-        uiDisplay.titleEdit->addItem(m_customLabel);
-    }
-
-    uiDisplay.titleEdit->addItem(i18n("None"));
-    uiDisplay.titleEdit->addItem(i18n("Default"));
-    uiDisplay.titleEdit->addItem(i18n("Full path"));
-    uiDisplay.titleEdit->setCurrentItem(configTitleText);
-    uiDisplay.titleEdit->completionBox()->setActivateOnSelect(true);
-    uiDisplay.titleEdit->setCompletionMode(KGlobalSettings::CompletionPopupAuto);
-    connect(uiDisplay.titleEdit->lineEdit(), SIGNAL(editingFinished()), this, SLOT(setTitleText()));
-
     const QList<int> iconSizes = QList<int>() << 16 << 22 << 32 << 48 << 64 << 128;
     uiDisplay.sizeSlider->setRange(0, iconSizes.size() - 1);
     uiDisplay.sizeSlider->setValue(iconSizes.indexOf(iconSize().width()));
 
     // Only add "Unsorted" as an option when we're showing an icon view, since the list view
     // doesn't allow the user to rearrange the icons.
-    if (m_iconView) {
-        uiDisplay.sortCombo->addItem(i18nc("Sort Icons", "Unsorted"), -1);
+    if (m_iconView && m_sortColumn == int(FolderView::Unsorted)) {
+        QAction *unsorted = m_actionCollection.action("unsorted");
+        if (unsorted) {
+            uiDisplay.sortCombo->addItem(KGlobal::locale()->removeAcceleratorMarker(unsorted->text()), unsorted->data());
+        }
     }
-    uiDisplay.sortCombo->addItem(KGlobal::locale()->removeAcceleratorMarker(m_actionCollection.action("sort_name")->text()), KDirModel::Name);
-    uiDisplay.sortCombo->addItem(KGlobal::locale()->removeAcceleratorMarker(m_actionCollection.action("sort_size")->text()), KDirModel::Size);
-    uiDisplay.sortCombo->addItem(KGlobal::locale()->removeAcceleratorMarker(m_actionCollection.action("sort_type")->text()), KDirModel::Type);
-    uiDisplay.sortCombo->addItem(KGlobal::locale()->removeAcceleratorMarker(m_actionCollection.action("sort_date")->text()), KDirModel::ModifiedTime);
 
-    uiDisplay.flowCombo->addItem(i18n("Top to Bottom, Left to Right"), QVariant::fromValue(IconView::TopToBottom));
-    uiDisplay.flowCombo->addItem(i18n("Top to Bottom, Right to Left"), QVariant::fromValue(IconView::TopToBottomRightToLeft));
-    uiDisplay.flowCombo->addItem(i18n("Left to Right, Top to Bottom"), QVariant::fromValue(IconView::LeftToRight));
-    uiDisplay.flowCombo->addItem(i18n("Right to Left, Top to Bottom"), QVariant::fromValue(IconView::RightToLeft));
+    addActionGroupToCombo(m_sortingGroup, uiDisplay.sortCombo);
+    addActionGroupToCombo(m_layoutGroup, uiDisplay.layoutCombo);
+    addActionGroupToCombo(m_alignmentGroup, uiDisplay.alignmentCombo);
 
     uiFilter.filterCombo->addItem(i18n("Show All Files"), QVariant::fromValue(ProxyModel::NoFilter));
     uiFilter.filterCombo->addItem(i18n("Show Files Matching"), QVariant::fromValue(ProxyModel::FilterShowMatches));
@@ -761,55 +754,19 @@ void FolderView::createConfigurationInterface(KConfigDialog *parent)
     uiDisplay.drawShadows->setChecked(m_drawShadows);
     uiDisplay.showPreviews->setChecked(m_showPreviews);
     uiDisplay.previewsAdvanced->setEnabled(m_showPreviews);
+    uiDisplay.sortDescending->setChecked(m_sortOrder == Qt::DescendingOrder);
+    uiDisplay.foldersFirst->setChecked(m_sortDirsFirst);
     uiDisplay.numLinesEdit->setValue(m_numTextLines);
     uiDisplay.colorButton->setColor(textColor());
 
-    for (int i = 0; i < uiDisplay.sortCombo->count(); i++) {
-       if (m_sortColumn == uiDisplay.sortCombo->itemData(i).toInt()) {
-           uiDisplay.sortCombo->setCurrentIndex(i);
-           break;
-       }
-    }
-
-    for (int i = 0; i < uiDisplay.flowCombo->count(); i++) {
-       if (m_flow == uiDisplay.flowCombo->itemData(i).value<IconView::Flow>()) {
-           uiDisplay.flowCombo->setCurrentIndex(i);
-           break;
-       }
-    }
-
-    for (int i = 0; i < uiFilter.filterCombo->count(); i++) {
-       if (m_filterType == uiFilter.filterCombo->itemData(i).value<ProxyModel::FilterMode>()) {
-           uiFilter.filterCombo->setCurrentIndex(i);
-           break;
-       }
-    }
+    setCurrentItem(uiDisplay.sortCombo, m_sortColumn);
+    setCurrentItem(uiDisplay.layoutCombo, m_layout);
+    setCurrentItem(uiDisplay.alignmentCombo, m_alignment);
+    setCurrentItem(uiLocation.titleCombo, m_labelType);
+    setCurrentItem(uiFilter.filterCombo, m_filterType);
 
     filterChanged(uiFilter.filterCombo->currentIndex());
 
-    // Hide the icon arrangement controls when we're not acting as a containment,
-    // since this option doesn't make much sense in the applet.
-    if (!isContainment()) {
-        uiDisplay.flowLabel->hide();
-        uiDisplay.flowCombo->hide();
-    }
-
-    connect(uiFilter.searchMimetype, SIGNAL(textChanged(QString)), pMimeModel, SLOT(setFilter(QString)));
-
-    parent->addPage(widgetLocation, i18nc("Title of the page that lets the user choose which location should the folderview show", "Location"), "folder");
-    parent->addPage(widgetDisplay, i18nc("Title of the page that lets the user choose how the folderview should be shown", "Display"), "preferences-desktop-display");
-    parent->addPage(widgetFilter, i18nc("Title of the page that lets the user choose how to filter the folderview contents", "Filter"), "view-filter");
-
-    connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
-    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
-    connect(uiLocation.showPlace, SIGNAL(toggled(bool)), uiLocation.placesCombo, SLOT(setEnabled(bool)));
-    connect(uiLocation.showCustomFolder, SIGNAL(toggled(bool)), uiLocation.lineEdit, SLOT(setEnabled(bool)));
-    connect(uiFilter.filterCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(filterChanged(int)));
-    connect(uiFilter.selectAll, SIGNAL(clicked(bool)), this, SLOT(selectAllMimetypes()));
-    connect(uiFilter.deselectAll, SIGNAL(clicked(bool)), this, SLOT(deselectAllMimeTypes()));
-    connect(uiDisplay.previewsAdvanced, SIGNAL(clicked()), this, SLOT(showPreviewConfigDialog()));
-    connect(uiDisplay.showPreviews, SIGNAL(toggled(bool)), uiDisplay.previewsAdvanced, SLOT(setEnabled(bool)));
-
     if (m_filterFilesMimeList.count()) {
         for (int i = 0; i < pMimeModel->rowCount(); i++) {
             const QModelIndex index = pMimeModel->index(i, 0);
@@ -821,14 +778,42 @@ void FolderView::createConfigurationInterface(KConfigDialog *parent)
         }
     }
 
-    connect(uiDisplay.titleEdit, SIGNAL(textChanged(QString)), parent, SLOT(settingsModified()));
-    connect(uiDisplay.flowCombo, SIGNAL(currentIndexChanged(int)), parent, SLOT(settingsModified()));
+    // Hide the icon arrangement controls when we're not acting as a containment,
+    // since this option doesn't make much sense in the applet.
+    if (!isContainment()) {
+        uiDisplay.layoutLabel->hide();
+        uiDisplay.layoutCombo->hide();
+        uiDisplay.alignmentLabel->hide();
+        uiDisplay.alignmentCombo->hide();
+    }
+
+    parent->addPage(widgetLocation, i18nc("Title of the page that lets the user choose which location should the folderview show", "Location"), "folder");
+    parent->addPage(widgetDisplay, i18nc("Title of the page that lets the user choose how the folderview should be shown", "Icons"), "preferences-desktop-icons");
+    parent->addPage(widgetFilter, i18nc("Title of the page that lets the user choose how to filter the folderview contents", "Filter"), "view-filter");
+
+    connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
+    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
+
+    connect(uiFilter.searchMimetype, SIGNAL(textChanged(QString)), pMimeModel, SLOT(setFilter(QString)));
+    connect(uiLocation.showPlace, SIGNAL(toggled(bool)), uiLocation.placesCombo, SLOT(setEnabled(bool)));
+    connect(uiLocation.showCustomFolder, SIGNAL(toggled(bool)), uiLocation.lineEdit, SLOT(setEnabled(bool)));
+    connect(uiLocation.titleCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(setTitleEditEnabled(int)));
+    connect(uiFilter.filterCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(filterChanged(int)));
+    connect(uiFilter.selectAll, SIGNAL(clicked(bool)), this, SLOT(selectAllMimetypes()));
+    connect(uiFilter.deselectAll, SIGNAL(clicked(bool)), this, SLOT(deselectAllMimeTypes()));
+    connect(uiDisplay.previewsAdvanced, SIGNAL(clicked()), this, SLOT(showPreviewConfigDialog()));
+    connect(uiDisplay.showPreviews, SIGNAL(toggled(bool)), uiDisplay.previewsAdvanced, SLOT(setEnabled(bool)));
+
+    connect(uiDisplay.layoutCombo, SIGNAL(currentIndexChanged(int)), parent, SLOT(settingsModified()));
+    connect(uiDisplay.alignmentCombo, SIGNAL(currentIndexChanged(int)), parent, SLOT(settingsModified()));
     connect(uiDisplay.sortCombo, SIGNAL(currentIndexChanged(int)), parent, SLOT(settingsModified()));
     connect(uiDisplay.sizeSlider, SIGNAL(valueChanged(int)), parent, SLOT(settingsModified()));
     connect(uiDisplay.showPreviews, SIGNAL(toggled(bool)), parent, SLOT(settingsModified()));
     connect(uiDisplay.lockInPlace, SIGNAL(toggled(bool)), parent, SLOT(settingsModified()));
     connect(uiDisplay.alignToGrid, SIGNAL(toggled(bool)), parent, SLOT(settingsModified()));
     connect(uiDisplay.clickToView, SIGNAL(toggled(bool)), parent, SLOT(settingsModified()));
+    connect(uiDisplay.sortDescending, SIGNAL(toggled(bool)), parent, SLOT(settingsModified()));
+    connect(uiDisplay.foldersFirst, SIGNAL(toggled(bool)), parent, SLOT(settingsModified()));
     connect(uiDisplay.numLinesEdit, SIGNAL(valueChanged(int)), parent, SLOT(settingsModified()));
     connect(uiDisplay.colorButton, SIGNAL(changed(QColor)), parent, SLOT(settingsModified()));
     connect(uiDisplay.drawShadows, SIGNAL(toggled(bool)), parent, SLOT(settingsModified()));
@@ -840,6 +825,8 @@ void FolderView::createConfigurationInterface(KConfigDialog *parent)
     connect(uiLocation.showDesktopFolder, SIGNAL(toggled(bool)), parent, SLOT(settingsModified()));
     connect(uiLocation.showActivity, SIGNAL(toggled(bool)), parent, SLOT(settingsModified()));
     connect(uiLocation.showPlace, SIGNAL(toggled(bool)), parent, SLOT(settingsModified()));
+    connect(uiLocation.titleCombo, SIGNAL(currentIndexChanged(int)), parent, SLOT(settingsModified()));
+    connect(uiLocation.titleEdit, SIGNAL(textChanged(QString)), parent, SLOT(settingsModified()));
     connect(uiLocation.showCustomFolder, SIGNAL(toggled(bool)), parent, SLOT(settingsModified()));
     connect(uiLocation.placesCombo, SIGNAL(currentIndexChanged(int)), parent, SLOT(settingsModified()));
     connect(uiLocation.lineEdit, SIGNAL(textChanged(QString)), parent, SLOT(settingsModified()));
@@ -893,16 +880,22 @@ void FolderView::configAccepted()
     const int sortColumn = uiDisplay.sortCombo->itemData(uiDisplay.sortCombo->currentIndex()).toInt();
     cg.writeEntry("sortColumn", sortColumn);
 
-    const IconView::Flow flow = uiDisplay.flowCombo->itemData(uiDisplay.flowCombo->currentIndex()).value<IconView::Flow>();
-    cg.writeEntry("flow", static_cast<int>(flow));
+    const Qt::SortOrder order = uiDisplay.sortDescending->isChecked() ? Qt::DescendingOrder : Qt::AscendingOrder;
+    cg.writeEntry("sortOrder", sortOrderEnumToString(order));
+
+    const bool dirsFirst = uiDisplay.foldersFirst->isChecked();
+    cg.writeEntry("sortDirsFirst", dirsFirst);
+
+    const IconView::Layout layout = uiDisplay.layoutCombo->itemData(uiDisplay.layoutCombo->currentIndex()).value<IconView::Layout>();
+    cg.writeEntry("layout", static_cast<int>(layout));
+
+    const IconView::Alignment alignment = uiDisplay.alignmentCombo->itemData(uiDisplay.alignmentCombo->currentIndex()).value<IconView::Alignment>();
+    cg.writeEntry("alignment", static_cast<int>(alignment));
 
     cg.writeEntry("alignToGrid", uiDisplay.alignToGrid->isChecked());
     cg.writeEntry("clickForFolderPreviews", uiDisplay.clickToView->isChecked());
     cg.writeEntry("iconsLocked", uiDisplay.lockInPlace->isChecked());
 
-    cg.writeEntry("blankLabel" , m_blankLabel);
-    cg.writeEntry("customLabel", m_customLabel);
-
     cg.writeEntry("url", url);
     cg.writeEntry("filterFiles", uiFilter.filterFilesPattern->text());
 
@@ -910,6 +903,17 @@ void FolderView::configAccepted()
     uiFilter.filterCombo->itemData(uiFilter.filterCombo->currentIndex()).value<ProxyModel::FilterMode>();
     cg.writeEntry("filter", static_cast<int>(filterMode));
 
+    const FolderView::LabelType labelType =
+    uiLocation.titleCombo->itemData(uiLocation.titleCombo->currentIndex()).value<FolderView::LabelType>();
+    QString customTitle;
+    if (labelType == FolderView::Custom) {
+        customTitle = uiLocation.titleEdit->text();
+    } else {
+        customTitle.clear();
+    }
+    cg.writeEntry("labelType", static_cast<int>(labelType));
+    cg.writeEntry("customLabel", customTitle);
+
     // Now, we have to iterate over all items (not only the filtered ones). For that reason we have
     // to ask the source model, not the proxy model.
     QStringList selectedItems;
@@ -1037,7 +1041,8 @@ void FolderView::updateIconViewState()
     m_iconView->setDrawShadows(m_drawShadows);
     m_iconView->setIconSize(iconSize());
     m_iconView->setTextLineCount(m_numTextLines);
-    m_iconView->setFlow(m_flow);
+    m_iconView->setLayout(m_layout);
+    m_iconView->setAlignment(m_alignment);
     m_iconView->setWordWrap(m_numTextLines > 1);
     m_iconView->setAlignToGrid(m_alignToGrid);
     m_iconView->setIconsMoveable(!m_iconsLocked);
@@ -1065,6 +1070,31 @@ void FolderView::addActions(AbstractItemView *view)
     view->addAction(m_actionCollection.action("del"));
 }
 
+void FolderView::addActionGroupToCombo(QActionGroup* group, QComboBox* combo)
+{
+    if (group && combo) {
+        foreach (QAction *action, group->actions()) {
+            combo->addItem(KGlobal::locale()->removeAcceleratorMarker(action->text()), action->data());
+        }
+    }
+}
+
+// We can not use QComboBox::findData() since in qt4, comparing QVariant's containing
+// enums (user types) fails even if the enums contained in the QVariant's are equal
+template <typename T>
+void FolderView::setCurrentItem(QComboBox* combo, T current)
+{
+    if (!combo)
+        return;
+
+    for (int i = 0; i < combo->count(); i++) {
+        if (current == combo->itemData(i).value<T>()) {
+            combo->setCurrentIndex(i);
+            break;
+        }
+    }
+}
+
 void FolderView::setupIconView()
 {
     if (m_iconView) {
@@ -1109,7 +1139,7 @@ void FolderView::setupIconView()
     QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(Qt::Vertical, this);
     layout->setContentsMargins(0, 0, 0, 0);
     layout->setSpacing(0);
-    if (m_label && !m_blankLabel) {
+    if (m_label && (m_labelType != FolderView::None)) {
         layout->addItem(m_label);
     }
     layout->addItem(m_iconView);
@@ -1183,14 +1213,6 @@ void FolderView::plasmaThemeChanged()
     }
 }
 
-void FolderView::networkStatusChanged(Solid::Networking::Status status)
-{
-    if (status == Solid::Networking::Connected && !m_url.isLocalFile() &&
-        m_url.protocol() != "desktop") {
-        refreshIcons();
-    }
-}
-
 /*
 // TODO Mark the cut icons as cut, but test performance!
 void FolderView::clipboardDataChanged()
@@ -1385,19 +1407,11 @@ void FolderView::setUrl(const KUrl &url)
     setAssociatedApplicationUrls(KUrl::List() << m_url);
 
     if (KProtocolInfo::protocolClass(m_url.protocol()) == ":local") {
-        disconnect(Solid::Networking::notifier(), 0, this, 0);
         m_dirLister->openUrl(m_url);
-    } else {
-        //If host is connected to the network and url is remote, list the remote files
-        connect(Solid::Networking::notifier(), SIGNAL(shouldConnect()), this,
-                SLOT(networkAvailable()), Qt::UniqueConnection);
-        //Check if network is not in the "Connected" state
-        if (Solid::Networking::status() == Solid::Networking::Connected) {
-             //FIXME: Remove the comment when KDE 4.9 development starts
-            //QString networkStatus(i18n("Network is not reachable"));
-            //showMessage(KIcon("dialog-warning"), networkStatus, Plasma::ButtonOk);
-            m_dirLister->openUrl(m_url);
-        }
+    } else if (Solid::Networking::status() == Solid::Networking::Connected) {
+        QString networkStatus(i18n("Network is not reachable"));
+        showMessage(KIcon("dialog-warning"), networkStatus, Plasma::ButtonOk);
+        m_dirLister->openUrl(m_url);
     }
 
     // Only parse desktop files when sorting if we're showing the desktop folder
@@ -1407,11 +1421,11 @@ void FolderView::setUrl(const KUrl &url)
 
 void FolderView::setAppletTitle()
 {
-    if (m_customLabel == "___EMPTY___") {
+    if (m_labelType == FolderView::None) {
         m_titleText.clear();
-    } else if (m_customLabel == "___FULL___") {
+    } else if (m_labelType == FolderView::FullPath) {
         m_titleText = m_url.path();
-    } else if (m_customLabel == "___DEFAULT___") {
+    } else if (m_labelType == FolderView::PlaceName) {
         if (m_url == KUrl("desktop:/")) {
             m_titleText = i18n("Desktop Folder");
         } else {
@@ -1442,9 +1456,9 @@ void FolderView::setAppletTitle()
     } else {
         m_titleText = m_customLabel;
     }
-    kDebug() << "WORKING WITH" << m_customLabel << "WE GOT" << m_titleText;
+    kDebug() << "WORKING WITH" << m_labelType << m_customLabel << "WE GOT" << m_titleText;
 
-    if (m_blankLabel) {
+    if (m_labelType == FolderView::None) {
         if (m_label) {
             m_label->hide();
         }
@@ -1464,7 +1478,7 @@ void FolderView::recreateLayout()
     QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(Qt::Vertical, this);
     layout->setContentsMargins(0, 0, 0, 0);
     layout->setSpacing(0);
-    if (!m_blankLabel) {
+    if (m_labelType != FolderView::None) {
         layout->addItem(m_label);
     }
     layout->addItem(m_iconView);
@@ -1562,12 +1576,37 @@ void FolderView::createActions()
         lockIcons->setChecked(m_iconsLocked);
         connect(lockIcons, SIGNAL(toggled(bool)), SLOT(toggleIconsLocked(bool)));
 
+        m_layoutGroup = new QActionGroup(this);
+        connect(m_layoutGroup, SIGNAL(triggered(QAction*)), SLOT(layoutChanged(QAction*)));
+        QAction *layoutRows = m_layoutGroup->addAction(i18nc("Arrange icons in", "Rows"));
+        QAction *layoutColumns = m_layoutGroup->addAction(i18nc("Arrange icons in", "Columns"));
+
+        layoutRows->setCheckable(true);
+        layoutRows->setData(QVariant::fromValue(IconView::Rows));
+
+        layoutColumns->setCheckable(true);
+        layoutColumns->setData(QVariant::fromValue(IconView::Columns));
+
+        m_alignmentGroup = new QActionGroup(this);
+        connect(m_alignmentGroup, SIGNAL(triggered(QAction*)), SLOT(alignmentChanged(QAction*)));
+        QAction *alignLeft = m_alignmentGroup->addAction(i18nc("Align icons", "Left"));
+        QAction *alignRight = m_alignmentGroup->addAction(i18nc("Align icons", "Right"));
+
+        alignLeft->setCheckable(true);
+        alignLeft->setData(QVariant::fromValue(IconView::Left));
+
+        alignRight->setCheckable(true);
+        alignRight->setData(QVariant::fromValue(IconView::Right));
+
+        KAction *unsorted = new KAction(i18nc("Sort icons", "Unsorted"), this);
+        unsorted->setData(int(FolderView::Unsorted));
+
         m_sortingGroup = new QActionGroup(this);
         connect(m_sortingGroup, SIGNAL(triggered(QAction*)), SLOT(sortingChanged(QAction*)));
-        QAction *sortByName = m_sortingGroup->addAction(i18nc("Sort icons", "By Name"));
-        QAction *sortBySize = m_sortingGroup->addAction(i18nc("Sort icons", "By Size"));
-        QAction *sortByType = m_sortingGroup->addAction(i18nc("Sort icons", "By Type"));
-        QAction *sortByDate = m_sortingGroup->addAction(i18nc("Sort icons", "By Date"));
+        QAction *sortByName = m_sortingGroup->addAction(i18nc("Sort icons by", "Name"));
+        QAction *sortBySize = m_sortingGroup->addAction(i18nc("Sort icons by", "Size"));
+        QAction *sortByType = m_sortingGroup->addAction(i18nc("Sort icons by", "Type"));
+        QAction *sortByDate = m_sortingGroup->addAction(i18nc("Sort icons by", "Date"));
 
         sortByName->setCheckable(true);
         sortByName->setData(int(KDirModel::Name));
@@ -1581,34 +1620,36 @@ void FolderView::createActions()
         sortByDate->setCheckable(true);
         sortByDate->setData(int(KDirModel::ModifiedTime));
 
-        m_sortingOrderGroup = new QActionGroup(this);
-        connect(m_sortingOrderGroup, SIGNAL(triggered(QAction*)), SLOT(sortingOrderChanged(QAction*)));
-        QAction *sortAscending = m_sortingOrderGroup->addAction(i18nc("Sort icons", "Ascending"));
-        QAction *sortDescending = m_sortingOrderGroup->addAction(i18nc("Sort icons", "Descending"));
-
-        sortAscending->setCheckable(true);
-        sortAscending->setData(QVariant::fromValue(Qt::AscendingOrder));
-
+        KAction *sortDescending = new KAction(i18nc("Sort icons", "Descending"), this);
         sortDescending->setCheckable(true);
-        sortDescending->setData(QVariant::fromValue(Qt::DescendingOrder));
+        sortDescending->setChecked(m_sortOrder == Qt::DescendingOrder);
+        connect(sortDescending, SIGNAL(toggled(bool)), SLOT(toggleSortDescending(bool)));
 
         KAction *dirsFirst = new KAction(i18nc("Sort icons", "Folders First"), this);
         dirsFirst->setCheckable(true);
         dirsFirst->setChecked(m_sortDirsFirst);
         connect(dirsFirst, SIGNAL(toggled(bool)), SLOT(toggleDirectoriesFirst(bool)));
 
-        QMenu *sortMenu = new QMenu(i18n("Sort Icons"));
+        QMenu *arrangeMenu = new QMenu(i18n("Arrange In"));
+        arrangeMenu->addAction(layoutRows);
+        arrangeMenu->addAction(layoutColumns);
+
+        QMenu *alignMenu = new QMenu(i18n("Align"));
+        alignMenu->addAction(alignLeft);
+        alignMenu->addAction(alignRight);
+
+        QMenu *sortMenu = new QMenu(i18n("Sort By"));
         sortMenu->addAction(sortByName);
         sortMenu->addAction(sortBySize);
         sortMenu->addAction(sortByType);
         sortMenu->addAction(sortByDate);
         sortMenu->addSeparator();
-        sortMenu->addAction(sortAscending);
         sortMenu->addAction(sortDescending);
-        sortMenu->addSeparator();
         sortMenu->addAction(dirsFirst);
 
         QMenu *iconsMenu = new QMenu;
+        iconsMenu->addMenu(arrangeMenu);
+        iconsMenu->addMenu(alignMenu);
         iconsMenu->addMenu(sortMenu);
         iconsMenu->addSeparator();
         iconsMenu->addAction(alignToGrid);
@@ -1626,14 +1667,20 @@ void FolderView::createActions()
 
         m_actionCollection.addAction("lock_icons", lockIcons);
         m_actionCollection.addAction("auto_align", alignToGrid);
+        m_actionCollection.addAction("sort_desc", sortDescending);
+        m_actionCollection.addAction("dirs_first", dirsFirst);
         m_actionCollection.addAction("icons_menu", iconsMenuAction);
+        m_actionCollection.addAction("layout_rows", layoutRows);
+        m_actionCollection.addAction("layout_columns", layoutColumns);
+        m_actionCollection.addAction("align_left", alignLeft);
+        m_actionCollection.addAction("align_right", alignRight);
+        m_actionCollection.addAction("unsorted", unsorted);
         m_actionCollection.addAction("sort_name", sortByName);
         m_actionCollection.addAction("sort_size", sortBySize);
         m_actionCollection.addAction("sort_type", sortByType);
         m_actionCollection.addAction("sort_date", sortByDate);
-        m_actionCollection.addAction("sort_ascending", sortAscending);
-        m_actionCollection.addAction("sort_descending", sortDescending);
 
+        updateFlowActionsState();
         updateSortActionsState();
     }
 }
@@ -1783,6 +1830,9 @@ void FolderView::toggleIconsLocked(bool locked)
     if (m_iconView) {
         m_iconView->setIconsMoveable(!locked);
     }
+    if (isUserConfiguring()) {
+        uiDisplay.lockInPlace->setChecked(locked);
+    }
 
     config().writeEntry("iconsLocked", locked);
     emit configNeedsSaving();
@@ -1795,6 +1845,9 @@ void FolderView::toggleAlignToGrid(bool align)
     if (m_iconView) {
         m_iconView->setAlignToGrid(align);
     }
+    if (isUserConfiguring()) {
+        uiDisplay.alignToGrid->setChecked(align);
+    }
 
     config().writeEntry("alignToGrid", align);
     emit configNeedsSaving();
@@ -1809,6 +1862,9 @@ void FolderView::toggleClickToViewFolders(bool enable)
     if (m_iconView) {
         m_iconView->setClickToViewFolders(enable);
     }
+    if (isUserConfiguring()) {
+        uiDisplay.clickToView->setChecked(enable);
+    }
 
     config().writeEntry("clickForFolderPreviews", enable);
     emit configNeedsSaving();
@@ -1816,14 +1872,34 @@ void FolderView::toggleClickToViewFolders(bool enable)
     m_delayedSaveTimer.start(5000, this);
 }
 
+void FolderView::toggleSortDescending(bool enable)
+{
+    m_sortOrder = enable ? Qt::DescendingOrder : Qt::AscendingOrder;
+
+    m_model->invalidate();
+    m_model->sort(m_sortColumn, m_sortOrder);
+    m_model->setDynamicSortFilter(true);
+    if (isUserConfiguring()) {
+        uiDisplay.sortDescending->setChecked(enable);
+    }
+
+    config().writeEntry("sortOrder", sortOrderEnumToString(m_sortOrder));
+    emit configNeedsSaving();
+
+    m_delayedSaveTimer.start(5000, this);
+}
+
 void FolderView::toggleDirectoriesFirst(bool enable)
 {
     m_sortDirsFirst = enable;
 
     m_model->setSortDirectoriesFirst(m_sortDirsFirst);
-    if (m_sortColumn != -1) {
+    if (m_sortColumn != int(FolderView::Unsorted)) {
         m_model->invalidate();
     }
+    if (isUserConfiguring()) {
+        uiDisplay.foldersFirst->setChecked(enable);
+    }
 
     config().writeEntry("sortDirsFirst", m_sortDirsFirst);
     emit configNeedsSaving();
@@ -1831,6 +1907,42 @@ void FolderView::toggleDirectoriesFirst(bool enable)
     m_delayedSaveTimer.start(5000, this);
 }
 
+void FolderView::layoutChanged(QAction* action)
+{
+    const IconView::Layout layout = action->data().value<IconView::Layout>();
+
+    if (layout != m_layout) {
+        m_layout = layout;
+        if (m_iconView) {
+            m_iconView->setLayout(m_layout);
+        }
+        if (isUserConfiguring()) {
+            setCurrentItem(uiDisplay.layoutCombo, m_layout);
+        }
+        config().writeEntry("layout", static_cast<int>(m_layout));
+        emit configNeedsSaving();
+        m_delayedSaveTimer.start(5000, this);
+    }
+}
+
+void FolderView::alignmentChanged(QAction* action)
+{
+    const IconView::Alignment alignment = action->data().value<IconView::Alignment>();
+
+    if (alignment != m_alignment) {
+        m_alignment = alignment;
+        if (m_iconView) {
+            m_iconView->setAlignment(m_alignment);
+        }
+        if (isUserConfiguring()) {
+            setCurrentItem(uiDisplay.alignmentCombo, m_alignment);
+        }
+        config().writeEntry("alignment", static_cast<int>(m_alignment));
+        emit configNeedsSaving();
+        m_delayedSaveTimer.start(5000, this);
+    }
+}
+
 void FolderView::sortingChanged(QAction *action)
 {
     const int column = action->data().toInt();
@@ -1840,24 +1952,22 @@ void FolderView::sortingChanged(QAction *action)
         m_model->sort(column, m_sortOrder);
         m_model->setDynamicSortFilter(true);
         m_sortColumn = column;
+        if (isUserConfiguring()) {
+            setCurrentItem(uiDisplay.sortCombo, m_sortColumn);
+        }
         config().writeEntry("sortColumn", m_sortColumn);
         emit configNeedsSaving();
         m_delayedSaveTimer.start(5000, this);
     }
 }
 
-void FolderView::sortingOrderChanged(QAction *action)
+void FolderView::updateFlowActionsState()
 {
-    const Qt::SortOrder order = action->data().value<Qt::SortOrder>();
-
-    if (order != m_sortOrder) {
-        m_model->invalidate();
-        m_model->sort(m_sortColumn, order);
-        m_model->setDynamicSortFilter(true);
-        m_sortOrder = order;
-        config().writeEntry("sortOrder", sortOrderEnumToString(m_sortOrder));
-        emit configNeedsSaving();
-        m_delayedSaveTimer.start(5000, this);
+    foreach (QAction *action, m_layoutGroup->actions()) {
+        action->setChecked(action->data().value<IconView::Layout>() == m_layout);
+    }
+    foreach (QAction *action, m_alignmentGroup->actions()) {
+        action->setChecked(action->data().value<IconView::Alignment>() == m_alignment);
     }
 }
 
@@ -1866,9 +1976,6 @@ void FolderView::updateSortActionsState()
     foreach (QAction *action, m_sortingGroup->actions()) {
         action->setChecked(action->data() == int(m_sortColumn));
     }
-    foreach (QAction *action, m_sortingOrderGroup->actions()) {
-        action->setChecked(action->data().value<Qt::SortOrder>() == m_sortOrder);
-    }
 }
 
 
@@ -1964,24 +2071,23 @@ void FolderView::indexesMoved(const QModelIndexList &indexes)
     Q_UNUSED(indexes)
 
     // If the user has rearranged the icons, the view is no longer sorted
-    if (m_sortColumn != -1) {
-        m_sortColumn = -1;
+    if (m_sortColumn != int(FolderView::Unsorted)) {
+        m_sortColumn = int(FolderView::Unsorted);
         m_model->setDynamicSortFilter(false);
         updateSortActionsState();
-        config().writeEntry("sortColumn", m_sortColumn);
-        emit configNeedsSaving();
 
         if (isUserConfiguring()) {
-            for (int i = 0; i < uiDisplay.sortCombo->count(); i++) {
-                if (m_sortColumn == uiDisplay.sortCombo->itemData(i).toInt()) {
-                    uiDisplay.sortCombo->setCurrentIndex(i);
-                    break;
-                }
+            QAction *unsorted = m_actionCollection.action("unsorted");
+            if (unsorted) {
+                uiDisplay.sortCombo->addItem(KGlobal::locale()->removeAcceleratorMarker(unsorted->text()), unsorted->data());
             }
+            setCurrentItem(uiDisplay.sortCombo, int(FolderView::Unsorted));
         }
-    }
 
-    m_delayedSaveTimer.start(5000, this);
+        config().writeEntry("sortColumn", m_sortColumn);
+        emit configNeedsSaving();
+        m_delayedSaveTimer.start(5000, this);
+    }
 }
 
 void FolderView::contextMenuRequest(QWidget *widget, const QPoint &screenPos)
@@ -2165,12 +2271,11 @@ QSizeF FolderView::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
               }
               break;
           case Plasma::Horizontal:
-          case Plasma::Vertical:
-                {
-                    const int iconSize = IconSize(KIconLoader::Panel);
-                    size = QSizeF(iconSize, iconSize);
-                    break;
-                }
+          case Plasma::Vertical: {
+              const int iconSize = IconSize(KIconLoader::Panel);
+              size = QSizeF(iconSize, iconSize);
+              break;
+          }
           default:
               break;
         }
@@ -2182,21 +2287,13 @@ QSizeF FolderView::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
 
 }
 
-void FolderView::setTitleText()
+void FolderView::setTitleEditEnabled(int index)
 {
-    QString text = uiDisplay.titleEdit->currentText();
-    if (text == i18n("None") || text.isEmpty()) {
-        m_customLabel.clear();
-        m_blankLabel = true;
-    } else if (text == i18n("Default")) {
-        m_customLabel = "___DEFAULT___";
-        m_blankLabel = false;
-    } else if (text == i18n("Full path")) {
-        m_customLabel = "___FULL___";
-        m_blankLabel = false;
+    if (uiLocation.titleCombo->itemData(index).value<FolderView::LabelType>() == FolderView::Custom) {
+        uiLocation.titleEdit->setEnabled(true);
+        uiLocation.titleEdit->setFocus();
     } else {
-        m_customLabel = text;
-        m_blankLabel = false;
+        uiLocation.titleEdit->setEnabled(false);
     }
 }
 
