@@ -82,7 +82,8 @@ IconView::IconView(QGraphicsWidget *parent)
       m_clickToViewFolders(true),
       m_showSelectionMarker(true),
       m_drawIconShrinked(false),
-      m_flow(layoutDirection() == Qt::LeftToRight ? LeftToRight : RightToLeft),
+      m_layout(Rows),
+      m_alignment(layoutDirection() == Qt::LeftToRight ? Left : Right),
       m_popupCausedWidget(0),
       m_dropOperation(0),
       m_dropActions(0),
@@ -209,10 +210,10 @@ bool IconView::wordWrap() const
     return m_wordWrap;
 }
 
-void IconView::setFlow(Flow flow)
+void IconView::setLayout(IconView::Layout layout)
 {
-    if (m_flow != flow) {
-        m_flow = flow;
+    if (m_layout != layout) {
+        m_layout = layout;
 
         // Schedule a full relayout
         if (!m_layoutBroken && m_validRows > 0) {
@@ -223,9 +224,28 @@ void IconView::setFlow(Flow flow)
     }
 }
 
-IconView::Flow IconView::flow() const
+IconView::Layout IconView::layout() const
 {
-    return m_flow;
+    return m_layout;
+}
+
+void IconView::setAlignment(IconView::Alignment alignment)
+{
+    if (m_alignment != alignment) {
+        m_alignment = alignment;
+
+        // Schedule a full relayout
+        if (!m_layoutBroken && m_validRows > 0) {
+            m_validRows = 0;
+            m_delayedLayoutTimer.start(10, this);
+            emit busy(true);
+        }
+    }
+}
+
+IconView::Alignment IconView::alignment() const
+{
+    return m_alignment;
 }
 
 void IconView::setAlignToGrid(bool on)
@@ -320,7 +340,7 @@ void IconView::updateGridSize()
     if (!containment || !containment->isContainment()) {
         getContentsMargins(&left, &top, &right, &bottom);
         QSize minSize = size + QSize(20 + left + right, 20 + top + bottom);
-        if (m_flow == LeftToRight || m_flow == RightToLeft) {
+        if (m_layout == Rows) {
             minSize.rwidth() += m_scrollBar->geometry().width();
         }
         setMinimumSize(minSize);
@@ -352,7 +372,7 @@ void IconView::updateGridSize()
                     const int row = qBound(0, topLeft.y() / oldAdjustedGridSize.height(), lastRow);
                     int col = topLeft.x() / oldAdjustedGridSize.width();
 
-                    if (m_flow == RightToLeft || m_flow == TopToBottomRightToLeft) {
+                    if (m_alignment == Right) {
                         col = lastCol - ((oldColCount - 1) - col);
                     }
 
@@ -375,7 +395,7 @@ void IconView::updateGridSize()
                         cr.bottom() - size.height() - margin);
                     int x = topLeft.x();
 
-                    if (m_flow == RightToLeft || m_flow == TopToBottomRightToLeft) {
+                    if (m_alignment == Right) {
                         x = cr.right() - qRound(scaleX * (cr.right() - x));
                     } else {
                         x = scaleX * x;
@@ -631,7 +651,7 @@ QPoint inline IconView::nextGridPosition(const QPoint &lastPos, const QSize &gri
     int margin = 10;
 
     QRect r = contentRect.adjusted(margin, margin, -margin, -margin);
-    if (m_flow == LeftToRight || m_flow == RightToLeft) {
+    if (m_layout == Rows) {
         if (layoutDirection() == Qt::LeftToRight) {
             r.adjust(0, 0, -m_scrollBar->geometry().width(), 0);
         } else {
@@ -639,7 +659,7 @@ QPoint inline IconView::nextGridPosition(const QPoint &lastPos, const QSize &gri
         }
     }
 
-    const int xOrigin = (m_flow == LeftToRight || m_flow == TopToBottom) ?
+    const int xOrigin = (m_alignment == Left) ?
                     r.left() :  r.right() - grid.width() + 1;
 
     if (lastPos.isNull()) {
@@ -648,30 +668,26 @@ QPoint inline IconView::nextGridPosition(const QPoint &lastPos, const QSize &gri
 
     QPoint pos = lastPos;
 
-    switch (m_flow) {
-    case LeftToRight:
-        pos.rx() += grid.width() + spacing;
-        if (pos.x() + grid.width() >= r.right()) {
-            pos.ry() += grid.height() + spacing;
-            pos.rx() = xOrigin;
+    if (m_layout == Rows) {
+        if (m_alignment == Left) {
+            pos.rx() += grid.width() + spacing;
+            if (pos.x() + grid.width() >= r.right()) {
+                pos.ry() += grid.height() + spacing;
+                pos.rx() = xOrigin;
+            }
+        } else { // Right
+            pos.rx() -= grid.width() + spacing;
+            if (pos.x() < r.left()) {
+                pos.ry() += grid.height() + spacing;
+                pos.rx() = xOrigin;
+            }
         }
-        break;
-
-    case RightToLeft:
-        pos.rx() -= grid.width() + spacing;
-        if (pos.x() < r.left()) {
-            pos.ry() += grid.height() + spacing;
-            pos.rx() = xOrigin;
-        }
-        break;
-
-    case TopToBottom:
-    case TopToBottomRightToLeft:
+    } else { // Columns
         pos.ry() += grid.height() + spacing;
         if (pos.y() + grid.height() >= r.bottom()) {
-            if (m_flow == TopToBottom) {
+            if (m_alignment == Left) {
                 pos.rx() += grid.width() + spacing;
-            } else { // RightToLeft
+            } else { // Right
                 pos.rx() -= grid.width() + spacing;
             }
             pos.ry() = r.top();
@@ -713,7 +729,7 @@ void IconView::layoutItems()
     const QSize grid = gridSize();
     int maxWidth = rect.width();
     int maxHeight = rect.height();
-    if (m_flow == LeftToRight || m_flow == RightToLeft) {
+    if (m_layout == Rows) {
         maxWidth -= m_scrollBar->geometry().width();
     }
     m_columns = columnsForWidth(maxWidth);
@@ -839,7 +855,7 @@ QRect IconView::adjustedContentsRect(const QSize &gridSize, int *rowCount, int *
     r.setHeight(r.height() - dy);
 
     // Take the origin into account
-    if (m_flow == RightToLeft || m_flow == TopToBottomRightToLeft) {
+    if (m_alignment == Right) {
         r.translate(dx, 0);
     }
 
@@ -857,7 +873,7 @@ void IconView::alignIconsToGrid()
     const Plasma::Containment *containment = qobject_cast<Plasma::Containment*>(parentWidget());
     if (!containment || !containment->isContainment()) {
         // Don't limit the max rows/columns in the scrolling direction
-        if (m_flow == LeftToRight || m_flow == RightToLeft) {
+        if (m_layout == Rows) {
             lastRow = INT_MAX;
         } else {
             lastCol = INT_MAX;
@@ -1576,7 +1592,7 @@ void IconView::resizeEvent(QGraphicsSceneResizeEvent *e)
     updateScrollBarGeometry();
 
     if (m_validRows > 0) {
-        if (m_flow == RightToLeft || m_flow == TopToBottomRightToLeft) {
+        if (m_alignment == Right) {
             // When the origin is the top-right corner, we need to shift all
             // the icons horizontally so they gravitate toward the right side.
             int dx = e->newSize().width() - e->oldSize().width();
@@ -1844,19 +1860,17 @@ void IconView::keyPressEvent(QKeyEvent *event)
         int hMultiplier = 1;
         int vMultiplier = 1;
 
-        switch (m_flow)        //Perform flow related calculations
-        {
-        case RightToLeft:
-            hMultiplier = -1;
-        case LeftToRight:        //Fall through because in both RightToLeft and LeftToRight, we move m_columns time vertically
+        //Perform flow related calculations
+        if (m_layout == Rows) {
             vMultiplier = m_columns;
-            break;
-        case TopToBottom:
+            if (m_alignment == Right) {
+                hMultiplier = -1;
+            }
+        } else { // Columns
             hMultiplier = m_rows;
-            break;
-        case TopToBottomRightToLeft:
-            hMultiplier = m_rows*(-1);
-            break;
+            if (m_alignment == Right) {
+                hMultiplier *= -1;
+            }
         }
 
         newItem = currentIndex.row() + hdirection*hMultiplier + vdirection*vMultiplier;
@@ -2425,7 +2439,7 @@ void IconView::dropEvent(QGraphicsSceneDragDropEvent *event)
     }
 
     // If the flow is vertical, check the top and bottom sides
-    if (m_flow == TopToBottom || m_flow == TopToBottomRightToLeft) {
+    if (m_layout == Columns) {
         if (boundingRect.top() < cr.top()) {
             delta.ry() += cr.top() - boundingRect.top();
         }
@@ -2490,7 +2504,7 @@ void IconView::changeEvent(QEvent *event)
         // a panel to the opposite edge, or when the user enables/disables panel autohide.
         bool widthChanged = int(m_margins[Plasma::LeftMargin] + m_margins[Plasma::RightMargin]) != int(left + right);
         bool heightChanged = int(m_margins[Plasma::TopMargin] + m_margins[Plasma::BottomMargin]) != int(top + bottom);
-        bool horizontalFlow = (m_flow == LeftToRight || m_flow == RightToLeft);
+        bool horizontalFlow = (m_layout == Rows);
         bool needRelayout = false;
 
         if ((horizontalFlow && widthChanged) || (!horizontalFlow && heightChanged)) {
@@ -2515,7 +2529,7 @@ void IconView::changeEvent(QEvent *event)
         } else {
             QPoint delta;
 
-            if (m_flow == LeftToRight || m_flow == TopToBottom) {
+            if (m_alignment == Left) {
                 // Gravitate toward the upper left corner
                 delta.rx() = int(left - m_margins[Plasma::LeftMargin]);
             } else {
@@ -2659,7 +2673,7 @@ void IconView::selectFirstOrLastIcon(bool firstIcon)
     int isFirst = firstIcon ? 1 : -1;    //Useful in calculations to decide whether to select First or Last icon
     QModelIndex toSelect;
 
-    if (m_flow == RightToLeft || m_flow == TopToBottomRightToLeft) {
+    if (m_alignment == Right) {
         dirn=-1;
     }
 
@@ -2828,7 +2842,7 @@ void IconView::timerEvent(QTimerEvent *event)
     else if (event->timerId() == m_delayedRelayoutTimer.timerId()) {
         m_delayedRelayoutTimer.stop();
 
-        bool horizontalFlow = (m_flow == LeftToRight || m_flow == RightToLeft);
+        bool horizontalFlow = (m_layout == Rows);
         if (m_layoutBroken) {
             // Relayout all icons that have ended up outside the view
             const QRect cr = contentsRect().toRect();
